/**
 * @file  Channel4ModuleApplication.cpp
 * @brief Implementierung der Klasse CChannel4ModuleApplication.
 */
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Channel4ModuleApplication.h"
#include <stdio.h>

// ##################################################################
/**
 * @brief Initialisieren der Membervariablen.
 *
 * @param id Nummer der Kanalklasse fuer z.B. Trace.
 */
void CChannel4ModuleApplication::InitClass(short channelID, short deviceID)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleApplication::InitClass [ %i, %i ]\n", __FILE__, __LINE__, channelID, deviceID);

	// Anzahl anstehender Datensaetze
	srand((unsigned int)time(NULL));
}

// ##################################################################
/**
 * @brief Destruktor.
 *
 */
CChannel4ModuleApplication::~CChannel4ModuleApplication() 
{
	while(m_records.size()) 
	{
		m_records.pop();
	}
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 */
void CChannel4ModuleApplication::GetErrorModule(short module, short error, char *text)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleApplication::GetErrorModule [ %i, %i, 0x%08X ]\n", __FILE__, __LINE__, module, error, text);

	// Ist IV_MODULE_APPLICATION ?
	if (module == IV_MODULE_APPLICATION)
	{

	}

	// Ermittlung des Fehlertextes an Basisklasse abgeben.
	CChannel4ModuleSystem::GetErrorModule(module, error, text);
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 */
void CChannel4ModuleApplication::GetErrorFunction(short module, short func, short error, char *text)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleApplication::GetErrorFunction [ %i, %i, %i, 0x%08X ]\n", __FILE__, __LINE__, module, func, error, text);

	if (module == IV_MODULE_APPLICATION)
	{
		// Sonderfehler fr Passwortschutz abhandeln.
		if (error == 28) 
		{
			// Die Modulfunktionen liefern den Fehlercode 28, wenn kein oder ein falsches Passwort verwendet wurde.
			// Es ist in diesem Fall zuerst das richtige Passwort fr die Kommunikation erforderlich.
			sprintf(text, "( %i ): Korrektes Passwort fr die Kommunikation erforderlich.", error);
			return;
		}
		else if (error == 29) 
		{
			// Die Modulfunktionen liefern den Fehlercode 29, wenn 3 mal ein falsches Passwort verwendet wurde.
			// Die Befehlsausfhrung ist danach fr 60 Sekunden blockiert.
			sprintf(text, "( %i ): Gert ist aufgrund von fehlerhaften Passworteingaben fr 60 Sekunden gesperrt.", error);
			return;
		}

		switch (func)
		{
		case IV_FUNCTION_CLEAR_RECORDS:
		case IV_FUNCTION_GET_RECORD:
		case IV_FUNCTION_SET_RECORD:
			switch(error) {
			case 1:sprintf(text, "( %i ): ", error); break;
			default: sprintf(text, "( %i ): Nicht eingetragener Funktionsfehler!", error); break;
			}
			break;
		case IV_FUNCTION_SHOW_MESSAGE:
			switch(error) {
			case 1:sprintf(text, "( %i ): Nachrichtentext zu lang fuer die Anzeige.", error); break;
			case 2:sprintf(text, "( %i ): Es wird bereits eine Nachricht angezeigt.", error); break;
			default: sprintf(text, "( %i ): Nicht eingetragener Funktionsfehler!", error); break;
			}
			break;
		case IV_FUNCTION_SET_VARIABLE:
		case IV_FUNCTION_GET_VARIABLE:
			switch(error) {
			case 1:sprintf(text, "( %i ): Der Majorbestandteil des Variablennamens ist unbekannt.", error); break;
			case 2:sprintf(text, "( %i ): Der Minorbestandteil des Variablennamens ist unbekannt.", error); break;
			case 3:sprintf(text, "( %i ): Der zu setzende Variablenwert ist zu lang.", error); break;
			default: sprintf(text, "( %i ): Nicht eingetragener Funktionsfehler!", error); break;
			}
			break;
		case IV_FUNCTION_SHOW_INFO:
			switch(error) {
			case 1:sprintf(text, "( %i ): Nachrichtentext zu lang fuer die Anzeige.", error); break;
			default: sprintf(text, "( %i ): Nicht eingetragener Funktionsfehler!", error); break;
			}
			break;
		default: sprintf(text, "( %i ): Nicht eingetragener Fehlerblock fr Funktion!", error); break;
		}
		
		return;
	}

	// Ermittlung des Fehlertextes an Basisklasse abgeben.
	CChannel4ModuleSystem::GetErrorFunction(module, func, error, text);
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int CChannel4ModuleApplication::Call(short module, short func, unsigned char *data, int &count)
{
	if (module == IV_MODULE_APPLICATION)
	{
		unsigned char status;
		switch (func)
		{
		case IV_FUNCTION_CLEAR_RECORDS: return ClearRecords(GetDeviceID());
		case IV_FUNCTION_SET_RECORD: return SetRecord(GetDeviceID(), status);
		case IV_FUNCTION_GET_RECORD: return GetRecord(GetDeviceID(), data, count, status);
		case IV_FUNCTION_SHOW_MESSAGE: return ShowMessage(GetDeviceID(), data[0], data[1], data[2], (char*)&data[3]);
		case IV_FUNCTION_SET_VARIABLE: return SetVariable(GetDeviceID(), (char*)&data[0], (char*)&data[strlen((char*)&data[0]) + 1]);
		case IV_FUNCTION_GET_VARIABLE: return GetVariable(GetDeviceID(), (char*)&data[0], (char*)&data[strlen((char*)&data[0]) + 1]);
		case IV_FUNCTION_SHOW_INFO: return ShowInfo(GetDeviceID(), data[0], data[1], data[2], data[3], *(short*)&data[6], (char*)&data[8]);
		case IV_FUNCTION_ENTRANCE2_IDENTIFICATION: 
			{
			char *id, *pin;
			char status[64]; 
			int statusLength;
			id = (char*)&data[strlen((char*)&data[0]) + 1];
			pin = &id[strlen(id) + 1];
			return Entrance2Identification(GetDeviceID(), (char*)&data[0], id, pin, status, statusLength); 
			}
		case IV_FUNCTION_ENTRANCE2_IDENTIFICATION_OK: return Entrance2IdentificationOK(GetDeviceID());
		case IV_FUNCTION_NOTIFY: return Notify(GetDeviceID(), data[0], *(short*)&data[1], (char*)&data[3], (char*)&data[3 + strlen((char*)&data[3])]);
		// Funktion nicht definiert
		default: return 0x0200; 
		}

	}

	return CChannel4ModuleSystem::Call(module, func, data, count);
}

//#########################################################
//##################### APPLICATION #######################
//#########################################################

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int CChannel4ModuleApplication::ClearRecords(short deviceID)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleApplication::ClearRecords [ %i ]\n", __FILE__, __LINE__, deviceID);

	int count = 0;
	
	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID)
	{
		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_APPLICATION, IV_FUNCTION_CLEAR_RECORDS, m_data, count)) != IV_ERROR_SUCCESS) 
		{
			// Fehler ausgeben
			TraceError(deviceID, IV_MODULE_APPLICATION, IV_FUNCTION_CLEAR_RECORDS);
			return m_result;
		}

		// Keine Antwortdaten erwartet
		if (count) 
		{
			Trace("%s(%i):WARNING(3): CChannel4ModuleApplication::ClearRecords [ Antwort enthaelt %i Byte an unnuetze Daten. ]\n", __FILE__, __LINE__, count);
		}
	}
	else 
	{
		unsigned char tmp[4];
		count = 0;
		
		while (m_records.size()) 
		{
			m_records.pop();
		}

		// Antwort des Befehls senden.
		if ((m_result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_APPLICATION, IV_FUNCTION_CLEAR_RECORDS, tmp, count)) != IV_ERROR_SUCCESS) 
		{
			
			Trace("%s(%i):ERROR(1): CChannel4ModuleApplication::ClearRecords [ Execute (%i) ]\n", __FILE__, __LINE__, m_result);
			return m_result;
		}
	}

	return m_result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceID Anzusprechende Geraetenummer.
*/
int CChannel4ModuleApplication::RestoreRecords(short deviceID)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleApplication::RestoreRecords [ %i ]\n", __FILE__, __LINE__, deviceID);

	int count = 0;

	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID)
	{
		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_APPLICATION, IV_FUNCTION_RESTORE_RECORDS, m_data, count)) != IV_ERROR_SUCCESS) 
		{
			// Fehler ausgeben
			TraceError(deviceID, IV_MODULE_APPLICATION, IV_FUNCTION_RESTORE_RECORDS);
			return m_result;
		}

		// Keine Antwortdaten erwartet
		if (count) 
		{
			Trace("%s(%i):WARNING(3): CChannel4ModuleApplication::RestoreRecords [ Antwort enthaelt %i Byte an unnuetze Daten. ]\n", __FILE__, __LINE__, count);
		}
	} 
	else 
	{
		unsigned char tmp[4];
		count = 0;

//		while (m_records.size()) 
//		{
//			m_records.pop();
//		}

		// Antwort des Befehls senden.
		if ((m_result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_APPLICATION, IV_FUNCTION_RESTORE_RECORDS, tmp, count)) != IV_ERROR_SUCCESS) 
		{
			Trace("%s(%i):ERROR(1): CChannel4ModuleApplication::RestoreRecords [ Execute (%i) ]\n", __FILE__, __LINE__, m_result);
			return m_result;
		}
	}

	return m_result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int CChannel4ModuleApplication::SetRecord(short deviceID, unsigned char &status)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleApplication::SetRecord [ %i, %i ]\n", __FILE__, __LINE__, deviceID, status);

	int count = 0;
	
	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID) 
	{
		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_APPLICATION, IV_FUNCTION_SET_RECORD, m_data, count)) != IV_ERROR_SUCCESS) 
		{
			// Fehler ausgeben
			TraceError(deviceID, IV_MODULE_APPLICATION, IV_FUNCTION_SET_RECORD);
			return m_result;
		}

		// OK
		if (count > (int)sizeof(unsigned char)) 
		{
			Trace("%s(%i):WARNING(1): CChannel4ModuleApplication::SetRecord [ Antwortparameter (%i Byte) ]\n", __FILE__, __LINE__, count);
		}
		
		switch (m_data[0]) 
		{
		case 0:
		case 2:
			status = 0;
			break;

		case MB_ANSW_ACK_ON_RECORD:        // Quittung erhalten (fuer Online-Datensatz) 
		case MB_ANSW_TIMEOUT_ACK_RECORD:   // Quittung kam nach Timeout (fuer Online-Datensatz) 
			status = 1;
			break;

		case MB_ANSW_OLD_ACK_ON_RECORD:    // wiederholt gesendete Quittung 
		case MB_ANSW_OLD_TIMEOUT_ACK_RECORD: // wiederholte Quittung nach Timeout 
			status = 2;
			break;

		case MB_ANSW_ACK_OFF_RECORD:       // Quittung erhalten (fuer Offline-Datensatz) 
			status = 3;
			break;

		case MB_ANSW_OLD_ACK_OFF_RECORD:   // wiederholt gesendete Quittung 
			status = 4;
			break;
		}
	} 
	else 
	{
		unsigned char data[1];
		
		if (m_records.size()) 
		{
			CRecord &record = m_records.front();

			switch (record.m_data[0]) 
			{
			case MB_ANSW_NO_RECORD:             // kein Datensatz vorhanden
				data[0] = 0;
				break;

			case MB_ANSW_ONLINE_RECORD:         // Online Datensatz 
			case MB_ANSW_OLD_ONLINE_RECORD:     // wiederholt gesendeter Online Datensatz 
				data[0] = MB_ANSW_ACK_ON_RECORD;
				break;

			case MB_ANSW_OFFLINE_RECORD:        // Offline Datensatz 
			case MB_ANSW_OLD_OFFLINE_RECORD:    // wiederholt gesendeter Offline Datensatz
				data[0] = MB_ANSW_ACK_OFF_RECORD;
				break;
			}
			
			count = sizeof(unsigned char);

			// Datensatz loeschen
			m_records.pop();
		} 
		else 
		{
			data[0] = 0;
			count = sizeof(unsigned char);
		}

		// Antwort des Befehls senden.
		if ((m_result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_APPLICATION, IV_FUNCTION_SET_RECORD, data, count)) != IV_ERROR_SUCCESS) 
		{	
			Trace("%s(%i):ERROR(1): CChannel4ModuleApplication::SetRecord [ Execute (%i) ]\n", __FILE__, __LINE__, m_result);
			return m_result;
		}
	}

	return m_result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int CChannel4ModuleApplication::GetRecord(short deviceID, unsigned char *buf, int &count, unsigned char &status)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleApplication::GetRecord [ %i, 0x08X, %i, %i ]\n", __FILE__, __LINE__, deviceID, buf, count, status);

	int i;
	
	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID) 
	{
		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_APPLICATION, IV_FUNCTION_GET_RECORD, m_data, count)) != IV_ERROR_SUCCESS) 
		{
			// Fehler ausgeben
			TraceError(deviceID, IV_MODULE_APPLICATION, IV_FUNCTION_GET_RECORD);
			return m_result;
		}

		// OK
		
		switch (m_data[0]) 
		{
		case MB_ANSW_NO_RECORD:             // kein Datensatz vorhanden
			status = 0;
			break;

		case MB_ANSW_ONLINE_RECORD:         // Online Datensatz 
			status = 1;
			break;

		case MB_ANSW_OLD_ONLINE_RECORD:     // wiederholt gesendeter Online Datensatz 
			status = 2;
			break;

		case MB_ANSW_OFFLINE_RECORD:        // Offline Datensatz 
			status = 3;
			break;

		case MB_ANSW_OLD_OFFLINE_RECORD:    // wiederholt gesendeter Offline Datensatz
			status = 4;
			break;
		}
		
		if (((status == 0) && (count > 1)) || (count > 243)) 
		{	
			return -IV_ERROR_PROCESSING_DLL;
		}


		for (i=0; i < count; i++) 
		{	
			buf[i] = m_data[i];
		}
	} 
	else 
	{
		unsigned char data[256];
		count = 0;
		
		if (m_records.size()) 
		{
			CRecord &record = m_records.front();

			// Daten kopieren
			data[0] = record.m_status;
			memcpy(&data[1], record.m_data, record.m_length);
			count = 1 + record.m_length;
		} 
		else 
		{
			data[count++] = MB_ANSW_NO_RECORD;
		}

		// Antwort des Befehls senden.
		if ((m_result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_APPLICATION, IV_FUNCTION_GET_RECORD, data, count)) != IV_ERROR_SUCCESS) 
		{	
			Trace("%s(%i):ERROR(1): CChannel4ModuleApplication::GetRecord [ Execute (%i) ]\n", __FILE__, __LINE__, m_result);
			return m_result;
		}
	}

	return m_result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int CChannel4ModuleApplication::ShowMessage(short deviceID, unsigned char time, unsigned char action, unsigned char audio, char *text)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleApplication::ShowMessage [ %i, %i, %i, %i, 0x%08X ]\n", __FILE__, __LINE__, deviceID, time, action, audio, text);

	int count = 0;
	
	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID) 
	{
		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);

		m_data[count] = time;
		count += sizeof(unsigned char);
		m_data[count] = action;
		count += sizeof(unsigned char);
		m_data[count] = audio;
		count += sizeof(unsigned char);

		// Namen uebernehmen
		count += sprintf((char*)&m_data[count], "%s", text) + 1;
		
		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_APPLICATION, IV_FUNCTION_SHOW_MESSAGE, m_data, count)) != IV_ERROR_SUCCESS) 
		{
			// Fehler ausgeben
			TraceError(deviceID, IV_MODULE_APPLICATION, IV_FUNCTION_SHOW_MESSAGE);
			return m_result;
		}

		// Keine Antwortdaten erwartet
		if (count) 
		{
			Trace("%s(%i):WARNING(3): CChannel4ModuleApplication::ShowMessage [ Antwort enthaelt %i Byte an unnuetze Daten. ]\n", __FILE__, __LINE__, count);
		}
	}
	else 
	{
		unsigned char tmp[4];
		count = 0;
		
		// Antwort des Befehls senden.
		if ((m_result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_APPLICATION, IV_FUNCTION_SHOW_MESSAGE, tmp, count)) != IV_ERROR_SUCCESS) 
		{	
			Trace("%s(%i):ERROR(1): CChannel4ModuleApplication::ShowMessage [ Execute (%i) ]\n", __FILE__, __LINE__, m_result);
			return m_result;
		}

		Trace("%s(%i):INFO(1): CChannel4ModuleApplication::ShowMessage [ %s ]\n", __FILE__, __LINE__, text);
	}

	return m_result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int CChannel4ModuleApplication::SetVariable(short deviceID, char *name, char *val)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleApplication::SetVariable [ %i, 0x08X, 0x%08X ]\n", __FILE__, __LINE__, deviceID, name, val);

	int count = 0;
	
	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID) 
	{
		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);

		// Namen uebernehmen
		count = sprintf((char*)m_data, "%s", name) + 1;
		// Wert uebernehmen
		count += sprintf((char*)&m_data[count], "%s", val) + 1;
		
		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_APPLICATION, IV_FUNCTION_SET_VARIABLE, m_data, count)) != IV_ERROR_SUCCESS) 
		{
			// Fehler ausgeben
			TraceError(deviceID, IV_MODULE_APPLICATION, IV_FUNCTION_SET_VARIABLE);
			return m_result;
		}

		// Keine Antwortdaten erwartet
		if (count) 
		{
			Trace("%s(%i):WARNING(3): CChannel4ModuleApplication::SetVariable [ Antwort enthaelt %i Byte an unnuetze Daten. ]\n", __FILE__, __LINE__, count);
		}
	} 
	else 
	{
		unsigned char tmp[4];
		count = 0;
		
		// Antwort des Befehls senden.
		if ((m_result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_APPLICATION, IV_FUNCTION_SET_VARIABLE, tmp, count)) != IV_ERROR_SUCCESS) 
		{	
			Trace("%s(%i):ERROR(1): CChannel4ModuleApplication::SetVariable [ Execute (%i) ]\n", __FILE__, __LINE__, m_result);
			return m_result;
		}

		Trace("%s(%i):INFO(1): CChannel4ModuleApplication::SetVariable [ %s = %s ]\n", __FILE__, __LINE__, name, val);
	}

	return m_result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int CChannel4ModuleApplication::GetVariable(short deviceID, char *name, char *val)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleApplication::GetVariable [ %i, 0x%08X, 0x%08X ]\n", __FILE__, __LINE__, deviceID, name, val);

	int count = 0;
	
	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID) 
	{
		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);

		// Namen uebernehmen
		count = sprintf((char*)m_data, "%s", name) + 1;
		
		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_APPLICATION, IV_FUNCTION_GET_VARIABLE, m_data, count)) != IV_ERROR_SUCCESS) 
		{
			// Fehler ausgeben
			TraceError(deviceID, IV_MODULE_APPLICATION, IV_FUNCTION_GET_VARIABLE);
			return m_result;
		}

		// Name+Wert als Antwortdaten erwartet
		if (count > (17 + 81)) 
		{
			Trace("%s(%i):WARNING(1): CChannel4ModuleApplication::GetVariable [ Antwortparameter (%i Byte) ]\n", __FILE__, __LINE__, count);
		}

		// Namen uebergehen
		count = (int)strlen(name) + 1;

		// OK
		strcpy(val, (char*)&m_data[count]);	
	
	} 
	else 
	{
		unsigned char tmp[17 + 81];
		memset(tmp, 0, sizeof(tmp));
		count = sprintf((char*)tmp, "%s", name) + 2;
		
		// Antwort des Befehls senden.
		if ((m_result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_APPLICATION, IV_FUNCTION_GET_VARIABLE, tmp, count)) != IV_ERROR_SUCCESS) 
		{	
			Trace("%s(%i):ERROR(1): CChannel4ModuleApplication::GetVariable [ Execute (%i) ]\n", __FILE__, __LINE__, m_result);
			return m_result;
		}

		Trace("%s(%i):INFO(1): CChannel4ModuleApplication::GetVariable [ %s ]\n", __FILE__, __LINE__, name);
	}

	return m_result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int CChannel4ModuleApplication::ShowInfo(short deviceID, unsigned char maschine, unsigned char time, unsigned char font, unsigned char audio, short flags, char *text)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleApplication::ShowInfo [ %i, %i, %i, %i, %i, 0x%04X, 0x%08X ]\n", __FILE__, __LINE__, deviceID, maschine, time, font, audio, flags, text);

	int count = 0;
// Maschinennummer (1 Byte)
// Time ist die Anzeigedauer. (1 Byte) (0 im Hindergrund uebernehmen, -1 staendig anzeigen)
// Schriftgroessennummer (1 Byte)
// Akkustisches Signal (1 Byte)
// 16bit reserve (2 Byte)
// Flags z.b. 0x0001 (2 Byte) fuer Antiproportionale Schrift
// Infotext
	
	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID) 
	{
		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);

		m_data[count] = maschine;
		count += sizeof(unsigned char);
		m_data[count] = time;
		count += sizeof(unsigned char);
		m_data[count] = font;
		count += sizeof(unsigned char);
		m_data[count] = audio;
		count += sizeof(unsigned char);
		*(short*)&m_data[count] = 0;
		count += sizeof(short);
		*(short*)&m_data[count] = flags;
		count += sizeof(short);

		// Namen uebernehmen
		count += sprintf((char*)&m_data[count], "%s", text) + 1;
		
		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_APPLICATION, IV_FUNCTION_SHOW_INFO, m_data, count)) != IV_ERROR_SUCCESS) 
		{
			// Fehler ausgeben
			TraceError(deviceID, IV_MODULE_APPLICATION, IV_FUNCTION_SHOW_INFO);
			return m_result;
		}

		// Keine Antwortdaten erwartet
		if (count) 
		{
			Trace("%s(%i):WARNING(3): CChannel4ModuleApplication::ShowInfo [ Antwort enthaelt %i Byte an unnuetze Daten. ]\n", __FILE__, __LINE__, count);
		}
	} 
	else 
	{
		unsigned char tmp[4];
		count = 0;
		
		// Antwort des Befehls senden.
		if ((m_result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_APPLICATION, IV_FUNCTION_SHOW_INFO, tmp, count)) != IV_ERROR_SUCCESS) 
		{	
			Trace("%s(%i):ERROR(1): CChannel4ModuleApplication::ShowInfo [ Execute (%i) ]\n", __FILE__, __LINE__, m_result);
			return m_result;
		}

		Trace("%s(%i):INFO(1): CChannel4ModuleApplication::ShowInfo [ %s ]\n", __FILE__, __LINE__, text);
	}

	return m_result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int CChannel4ModuleApplication::Entrance2Identification(short deviceID, char *TM, char *Id, char *Pin, char *status, int &statusLength)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleApplication::Entrance2Identification [ %i, %s, %s, %s ]\n", __FILE__, __LINE__, deviceID, TM, Id, Pin);

	int count = 0;
	
	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID) 
	{
		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);


		// Namen uebernehmen
		count = sprintf((char*)&m_data[count], TM) + 1;
		count += sprintf((char*)&m_data[count], Id) + 1;
		count += sprintf((char*)&m_data[count], Pin) + 1;
		
		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_APPLICATION, IV_FUNCTION_ENTRANCE2_IDENTIFICATION, m_data, count)) != IV_ERROR_SUCCESS) 
		{
			// Fehler ausgeben
			TraceError(GetDeviceIDRemote(), IV_MODULE_APPLICATION, IV_FUNCTION_ENTRANCE2_IDENTIFICATION);
			return m_result;
		}

		// Keine Antwortdaten erwartet
		if (count) 
		{
			Trace("%s(%i):WARNING(1): CChannel4ModuleApplication::Entrance2Identification [ Antwortparameter (%i Byte) ]\n", __FILE__, __LINE__, count);
		}

		count = 0;

		if ((m_result = Execute(IV_CONTEXT_SLAVE, IV_MODULE_APPLICATION, IV_FUNCTION_ENTRANCE2_IDENTIFICATION_OK, m_data, count, IV_PENDING_TRIALS(GetTimeout()))) != IV_ERROR_SUCCESS) 
		{
			// Fehler ausgeben
			TraceError(GetDeviceIDRemote(), IV_MODULE_APPLICATION, IV_FUNCTION_ENTRANCE2_IDENTIFICATION_OK);
			return m_result;
		}

		if (count == 0) 
		{
			Trace("%s(%i):WARNING(1): CChannel4ModuleApplication::Entrance2Identification [ Aktiver Antwortparameter (%i Byte) ]\n", __FILE__, __LINE__, count);
		}

		// Wert von Status uebernehmen
		int i;
		for (i=0; (i < count) && (i < statusLength); i++) 
		{
			status[i] = m_data[i];
		}
		// Uebergebene Laenge mitteilen
		statusLength = i - 1;
	} 
	else 
	{
		unsigned char tmp[64];
		count = 0;
		
		// Antwort des Befehls senden.
		if ((m_result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_APPLICATION, IV_FUNCTION_ENTRANCE2_IDENTIFICATION, tmp, count)) != IV_ERROR_SUCCESS) 
		{	
			Trace("%s(%i):ERROR(1): CChannel4ModuleApplication::Entrance2Identification [ Execute (%i) ]\n", __FILE__, __LINE__, m_result);
			return m_result;
		}

		// Aktuellen Status als aktive Antwort senden
		count = sprintf((char*)tmp, "%s", "Keiner") + 1;

		// Aktive Antwort des Befehls senden.
		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_APPLICATION, IV_FUNCTION_ENTRANCE2_IDENTIFICATION_OK, tmp, count)) != IV_ERROR_SUCCESS) 
		{	
			Trace("%s(%i):ERROR(1): CChannel4ModuleApplication::Entrance2Identification [ Execute (%i) ]\n", __FILE__, __LINE__, m_result);
			return m_result;
		}
	}

	return m_result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int CChannel4ModuleApplication::Entrance2IdentificationOK(short deviceID)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleApplication::Entrance2IdentificationOK [ %i ]\n", __FILE__, __LINE__, deviceID);

	int count = 0;
	
	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID)
	{
		Trace("%s(%i):ERROR(1): CChannel4ModuleApplication::Entrance2IdentificationOK \n", __FILE__, __LINE__);
	} 
	else 
	{
		unsigned char tmp[4];
		count = 0;
		
		// Antwort des Befehls senden.
		if ((m_result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_APPLICATION, IV_FUNCTION_ENTRANCE2_IDENTIFICATION_OK, tmp, count)) != IV_ERROR_SUCCESS) 
		{	
			Trace("%s(%i):ERROR(1): CChannel4ModuleApplication::Entrance2IdentificationOK [ Execute (%i) ]\n", __FILE__, __LINE__, m_result);
			return m_result;
		}

		Trace("%s(%i):ERROR(1): CChannel4ModuleApplication::Entrance2IdentificationOK [ %s ]\n", __FILE__, __LINE__, m_result);
	}

	return m_result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceID Anzusprechende Geraetenummer.
*/
int CChannel4ModuleApplication::Entrance2OnlineAction(short deviceID, char *TM, unsigned int mask, unsigned char type, unsigned char duration)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleApplication::Entrance2OnlineAction [ %i, 0x08X, 0x%08X, %u, %u ]\n", __FILE__, __LINE__, deviceID, TM, mask, type, duration);

	int count = 0;

	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID)
	{
		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);

		// Nummer des Tuermodul uebernehmen
		count = sprintf((char*)m_data, "%s", TM) + 1;
		// Weiteren Werte uebernehmen
		*(unsigned int*)&m_data[count] = mask;
		count += sizeof(unsigned int);
		m_data[count] = type;
		count += sizeof(unsigned char);
		m_data[count] = duration;
		count += sizeof(unsigned char);

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_APPLICATION, IV_FUNCTION_ENTRANCE2_ONLINE_ACTION, m_data, count)) != IV_ERROR_SUCCESS) 
		{
			// Fehler ausgeben
			TraceError(deviceID, IV_MODULE_APPLICATION, IV_FUNCTION_ENTRANCE2_ONLINE_ACTION);
			return m_result;
		}

		// Keine Antwortdaten erwartet
		if (count) 
		{
			Trace("%s(%i):WARNING(3): CChannel4ModuleApplication::Entrance2OnlineAction [ Antwort enthaelt %i Byte an unnuetze Daten. ]\n", __FILE__, __LINE__, count);
		}
	} 
	else 
	{
		unsigned char tmp[4];
		count = 0;

		// Antwort des Befehls senden.
		if ((m_result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_APPLICATION, IV_FUNCTION_ENTRANCE2_ONLINE_ACTION, tmp, count)) != IV_ERROR_SUCCESS) 
		{
			Trace("%s(%i):ERROR(1): CChannel4ModuleApplication::Entrance2OnlineAction [ Execute (%i) ]\n", __FILE__, __LINE__, m_result);
			return m_result;
		}

		Trace("%s(%i):INFO(1): CChannel4ModuleApplication::Entrance2OnlineAction [ %s ]\n", __FILE__, __LINE__, TM);
	}

	return m_result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int CChannel4ModuleApplication::Notify(short deviceID, unsigned char type, int priority, char *timestamp, char *params)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleApplication::Notify [ %i, %i, %i, %s ]\n", __FILE__, __LINE__, deviceID, type, priority, params);

	int count = 0;
	
	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID)
	{
		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);

		// Benachrichtigungstyp uebernehmen
		m_data[count] = type;
		count += sizeof(unsigned char);
		// Prioritaet eintragen
		*(short*)&m_data[count] = priority;
		count += sizeof(short);
		// Zeitstempel
		if (timestamp == NULL) 
		{
			time_t tm_time;
			struct tm *ptm;
			tm_time = time(NULL);
			ptm = localtime(&tm_time);
			count += sprintf((char*)&m_data[count], "%02i%02i-%02i-%02i %02i:%02i:%02i:%03i", 
				(ptm->tm_year + 1900) / 100,
				(ptm->tm_year + 1900) % 100,
				ptm->tm_mon + 1,
				ptm->tm_mday,
				ptm->tm_hour,
				ptm->tm_min,
				ptm->tm_sec, 0) + 1;
		}
		else 
		{
			count += sprintf((char*)&m_data[count], "%s", timestamp) + 1;
		}
		// Leerstring
		count += sprintf((char*)&m_data[count], "%s", params) + 1;

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_APPLICATION, IV_FUNCTION_NOTIFY, m_data, count)) != IV_ERROR_SUCCESS)
		{
			// Fehler ausgeben
			TraceError(deviceID, IV_MODULE_APPLICATION, IV_FUNCTION_NOTIFY);
			return m_result;
		}

		// OK
		if (count > (int)sizeof(int)) 
		{
			Trace("%s(%i):WARNING(1): CChannel4ModuleApplication::Notifiy [ Antwortparameter (%i Byte) ]\n", __FILE__, __LINE__, count);
		}
	}
	else 
	{
		unsigned char tmp[4];
		count = sizeof(tmp);
		tmp[0] = 1;
		
		switch (type) 
		{
		case IV_VALUE_NOTIFY_RECORD: 
			{
			CNotifyRecord nr;

			// Hat das Kanalobjekt den Aktiven Zustand?
			if (GetActiveState() < is_active) 
			{
				// Funktion derzeit nicht ausfhrbar.
				return 0x0001;
			}
			//TRACE("ActiveState: %i\n", GetActiveState());

			if (m_CBNotifyRecord) 
			{
				int pos = 0;

				// Aktuelle Kanalnummer eintragen
				nr.channelID = GetID();

				// Aktuelle RemoteBusnummer eintragen
				nr.deviceID = GetDeviceIDRemote();

				// Meldungstypen uebernehmen
				nr.type = type;
				pos += sizeof(unsigned char);

				// Meldungsprioritaet uebernehmen
				nr.priority = priority;
				pos += sizeof(unsigned short);

				// Zeitstempel der Meldungserzeugung uebernehmen
				pos += sprintf(nr.timestamp, "%s", timestamp);

				// Aktuellen Zeitstempel fuer Nachrichtenannahme halten
				nr.dt = time(NULL);

				// Informationsstring aufbauen
				sprintf(nr.infoString, "%u\r%u\r", m_hwInfo.typeDevice, (unsigned int)m_serial);

				// Meldung in Warteschlange stellen.
				if (m_CBNotifyRecord(nr) != IV_ERROR_SUCCESS) 
				{
					// Die Annahme der Aktiven Datensatzmeldung schlug fehl (ggf. nicht aktiv).
					return 0x0002;
				}

				Trace("%s(%i):INFO(1): CChannel4ModuleApplication::Notify [ %i ]\n", __FILE__, __LINE__, GetID());
			} 
			else 
			{
				// Die Annahme der Aktiven Datensatzmeldung ist nicht aktiv.
				return 0x0002;
			}
			break;
			}

		default:
			Trace("%s(%i):INFO(1): CChannel4ModuleApplication::Notify [ Unknown type %i ]\n", __FILE__, __LINE__, type);
			break;
		}


		// Antwort des Befehls senden.
		if ((m_result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_APPLICATION, IV_FUNCTION_NOTIFY, tmp, count)) != IV_ERROR_SUCCESS) 
		{	
			Trace("%s(%i):ERROR(1): CChannel4ModuleApplication::Notify [ Execute (%i) ]\n", __FILE__, __LINE__, m_result);
			return m_result;
		}
	}

	return m_result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int CChannel4ModuleApplication::NotifyRecord(short deviceID, int priority)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleApplication::NotifyRecord [ %i, %i ]\n", __FILE__, __LINE__, deviceID, priority);

	return Notify(deviceID, IV_VALUE_NOTIFY_RECORD, priority, NULL, "");
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceID Anzusprechende Geraetenummer.
* @param type 
0		Standardschrift(Sicherstellung der Abwrtskompatibilitt)
1		16 Pixel (7Anzeigezeilen)
2		16 Pixel,feste breite(7Zeilen)
3		19 Pixel(6Anzeigezeilen)
4		19 Pixel,feste breite(6Zeilen)
5		21 Pixel(5Anzeigezeilen)
6		21 Pixel,feste breite(5Zeilen)
*/
int CChannel4ModuleApplication::SetTypeFont(short deviceID, unsigned char type)
{
	Trace("%s(%i):INFO(3): CChannel4ModuleApplication::SetTypeFont [ %i, %i ]\n", __FILE__, __LINE__, deviceID, type);

	int count = 0;

	// Funktion in Remoteklasse aufrufen?
	if (GetDeviceID() != deviceID)
	{
		// Umgebung zuruecksetzen
		Reset();

		// Remoteadresse setzen.
		SetDeviceIDRemote(deviceID);

		// Typewert uebernehmen
		count = sizeof(unsigned char);
		// Wert uebernehmen
		*m_data = type;

		if ((m_result = Execute(IV_CONTEXT_MASTER, IV_MODULE_APPLICATION, IV_FUNCTION_SET_TYPE_FONT, m_data, count)) != IV_ERROR_SUCCESS) 
		{
			// Fehler ausgeben
			TraceError(deviceID, IV_MODULE_APPLICATION, IV_FUNCTION_SET_TYPE_FONT);
			return m_result;
		}

		// Keine Antwortdaten erwartet
		if (count) 
		{
			Trace("%s(%i):WARNING(3): CChannel4ModuleApplication::SetTypeFont [ Antwort enthaelt %i Byte an unnuetze Daten. ]\n", __FILE__, __LINE__, count);
		}
	} 
	else 
	{
		unsigned char tmp[4];
		count = 0;

		// Antwort des Befehls senden.
		if ((m_result = Execute(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_APPLICATION, IV_FUNCTION_SET_TYPE_FONT, tmp, count)) != IV_ERROR_SUCCESS) 
		{
			Trace("%s(%i):ERROR(1): CChannel4ModuleApplication::SetTypeFont [ Execute (%i) ]\n", __FILE__, __LINE__, m_result);
			return m_result;
		}
	}

	return m_result;
}

