#ifndef _CHANNEL4_MODULE_APPLICATION_H_
#define _CHANNEL4_MODULE_APPLICATION_H_

// Basisklasse einbinden
#include "Channel4ModuleSystem.h"

// MODULE_APPLICATION( 2 )

#define IV_MODULE_APPLICATION                    0x0002

// Fehler:
// 1 -> Modul unbekannt
// 2 -> Funktion unbekannt
// 3 -> Zu wenige Parameter
// 4 -> Zu viele Parameter

// Clear: Alle Datensaetze loeschen. CLEAR_RECORDS( 0 )
#define IV_FUNCTION_CLEAR_RECORDS                0x0000
// Set: Datensatz quittieren. SET_RECORD( 1 )
#define IV_FUNCTION_SET_RECORD                   0x0001
// Get: Datensatz hohlen. GET_RECORD( 2 )
#define IV_FUNCTION_GET_RECORD                   0x0002

// Datensatzanfrage
//#define MB_CMD_READ_RECORD              0x40  /* Datensatz senden */

#define MB_ANSW_NO_RECORD               0xC0  /* kein Datensatz vorhanden */
#define MB_ANSW_OFFLINE_RECORD          0xC1  /* Offline Datensatz */
#define MB_ANSW_OLD_OFFLINE_RECORD      0xC3  /* wiederholt gesendeter Offline Datensatz */
#define MB_ANSW_ONLINE_RECORD           0xE1  /* Online Datensatz */
#define MB_ANSW_OLD_ONLINE_RECORD       0xE3  /* wiederholt gesendeter Online Datensatz */

// Datensatzquittierung
//#define MB_CMD_ACK_RECORD               0x48  /* Quittung fuer Datensatz */

#define MB_ANSW_ACK_OFF_RECORD	        0xC8  /* Quittung erhalten (fuer Offline-Datensatz) */
#define MB_ANSW_OLD_ACK_OFF_RECORD      0xCA  /* wiederholt gesendete Quittung */
#define MB_ANSW_ACK_ON_RECORD           0xE8  /* Quittung erhalten (fuer Online-Datensatz) */
#define MB_ANSW_OLD_ACK_ON_RECORD       0xEA  /* wiederholt gesendete Quittung */
#define MB_ANSW_TIMEOUT_ACK_RECORD      0xE9  /* Quittung kam nach Timeout (fuer Online-Datensatz) */
#define MB_ANSW_OLD_TIMEOUT_ACK_RECORD  0xEB  /* wiederholte Quittung nach Timeout */

// Datensatzloeschbefehl
//#define MB_CMD_DEL						0x49  /* Loescht alle Datensaetze. */



// Set: Message SHOW_MESSAGE( 3 )
#define IV_FUNCTION_SHOW_MESSAGE                 0x0003
// (UIN8(Time)unsigned char(Action)unsigned char(Audio)Message\0)
// -> ()
// Time ist die anzuzeigende Dauer
// Action ist die durchzufuehrende Aktion (unbenutzt und immer 0)
// Audio ist das zu verwendende akustische Signal
// Message ist der eigentliche Nachrichtentext

// Fehler:
// 1 -> Nachrichtentext zu lang fuer die Anzeige.
// 2 -> Es wird bereits eine Nachricht angezeigt.


// Set: Variable (Setup) SET_VARIABLE( 4 )
#define IV_FUNCTION_SET_VARIABLE                 0x0004
// Get: Variable (Setup) GET_VARIABLE( 5 )
#define IV_FUNCTION_GET_VARIABLE                 0x0005

// Der Name einer Variablen setzt sich auf folgende Art zusammen:
// MAJOR_NAME.MINOR_NAME
// Der Majorname einer Setupvariablen ist immer SETUP.
// Fehlernummer:
// 1 = MAJOR_NAME ist unbekannt
// 2 = MINOR_NAME ist unbekannt
// 3 = Lnge des Variablenwertes ist zu lang.

#define IV_FUNCTION_SHOW_INFO                    0x0006
// Maschinennummer (1 Byte)
// Time ist die Anzeigedauer. (1 Byte) (0 im Hindergrund uebernehmen, -1 staendig anzeigen)
// Schriftgroessennummer (1 Byte)
// Akkustisches Signal (1 Byte)
// 16bit reserve (2 Byte)
// Flags z.b. 0x0001 (2 Byte) fuer Antiproportionale Schrift
// Infotext

#define IV_FUNCTION_ENTRANCE2_IDENTIFICATION     0x0007
// Alle Parameter mit fester Laenge. Linksausrichtung.
// param1: 3 Zeichen Tuermodul
// param2: 20 Zeichen Ausweis
// param3: 8 Zeichen Pin

// Aktive Antwort: Status der Zutrittspruefung als Parameter
#define IV_FUNCTION_ENTRANCE2_IDENTIFICATION_OK  0x0008

// Benachrichtigung ueber ein Ereignis
#define IV_FUNCTION_NOTIFY                       0x0009

// Mindestens ein Datensatz liegt vor.
#define IV_VALUE_NOTIFY_RECORD                   0x00

class CNotifyRecord 
{
public:
	CNotifyRecord() {
		channelID = 0;
		deviceID = 0;
		type = 0;
		priority = 0;
		memset(timestamp, 0, sizeof(timestamp));
		dt = 0;
		memset(infoString, 0, sizeof(infoString));
		erase = 0;
	}

public:
	int channelID;      // Kanalnummer des Geraetes
	int deviceID;       // Busnummer des Geraetes
	unsigned char type; // Meldungstyp
	short priority;     // Prioritaet des Geraetes
	char timestamp[24]; // Zeitstempel der Meldungserzeugung (Format: "YYYY-MM-DD hh:mm:ss:mis")
	time_t dt;          // Zeitstempel der Annahme
	char infoString[64];// Zusatzinformationen fuer Anwendung mit \r getrennt
	int erase;          // Wie der Dateninhalt zu handhaben ist. 0 -> Hinzufuegen, 1 -> Entfernen

public:
	bool operator < (const CNotifyRecord &other) const
	{
		return (priority < other.priority);
	}
};

class CRecord
{
public:
	CRecord() 
	{
		m_priority = 0;
		m_status = 0;
		memset(&m_timestamp, 0, sizeof(tm));
		m_delay = 0;
		m_length = 0;
		memset(m_data, 0, sizeof(m_data));
	}
	int m_priority;
	int m_status;
	struct tm m_timestamp;
	int m_delay;
	int m_length;
	unsigned char m_data[256];
};


#define IV_FUNCTION_ENTRANCE2_ONLINE_ACTION      0x000A
// Alle Parameter mit fester Laenge. Linksausrichtung.
// param1: 3 Zeichen Tuermodul
// param2: 4Byte Bitmaske: Bit0: Buzzer, Bit1: LED Grn, Bit2: LED Rot, Bit3: Relais 1, Bit4: Relais 2
// param3: 1Byte Typ. 0=Dauer Aus, 1=Dauer Ein, 2=Blinken (600ms an/aus)
// param4: 1Byte Dauer. Bei Typ = 2 ist 0=Dauernd ein oder 1 - 40 Sekunden (maximal 48Sekunden da 255*200ms)

// Restore: Datensatzzeiger auf ersten gueltigen Datensatz zurruecksetzen.
#define IV_FUNCTION_RESTORE_RECORDS             0x000B

// Den Type des zu verwendenden Zeichensatzes fuer Messages und Infos festsetzen
#define IV_FUNCTION_SET_TYPE_FONT               0x000C

// Kontainerklassen fuer Prioritaetswarteschlange importieren.
#include <deque>
#include <queue>

typedef std::deque<CRecord> record_deque;
typedef std::queue<CRecord> record_queue;

class CChannel4ModuleApplication : public CChannel4ModuleSystem
{
public:
	/// Konstruktor fuer Initialisierung der Klasse.
	CChannel4ModuleApplication()  : CChannel4ModuleSystem(IV_DEFAULT_CHANNELID, IV_DEFAULT_DEVICEID)
	{ 
		InitClass(); 
	}
	CChannel4ModuleApplication(short channelID, short deviceID) : CChannel4ModuleSystem(channelID, deviceID) 
	{ 
		InitClass(channelID, deviceID); 
	}
	virtual void InitClass(short channelID = IV_DEFAULT_CHANNELID, short deviceID = IV_DEFAULT_DEVICEID);
	/// Destruktor
	virtual ~CChannel4ModuleApplication();

public:
	// Setzen der CallBack-Funktion fuer Datensatzbenachrichtigung
	void SetCBNotifyRecord(int (*CBNotifyRecord)(CNotifyRecord&))
	{
		m_CBNotifyRecord = CBNotifyRecord;
	}

protected:
	int (*m_CBNotifyRecord)(CNotifyRecord&);


public:
	// Gibt Modulfehler zurueck.
	virtual void GetErrorModule(short module, short error, char *text);
	// Gibt Funktionsfehlertext zurueck.
	virtual void GetErrorFunction(short module, short func, short error, char *text);

	// MODULE_APPLICATION
	record_queue m_records;

	// Datensaetze loeschen
	int ClearRecords(short deviceID);

	// Datensatzzeiger ruecksetzen
	int RestoreRecords(short deviceID);

	// Datensatz quittieren/anfordern
	int SetRecord(short deviceID, unsigned char &status);
	int GetRecord(short deviceID, unsigned char *buf, int &count, unsigned char &status);

	// Displaynachicht senden
	int ShowMessage(short deviceID, unsigned char time, unsigned char action, unsigned char audio, char *text);

	// Variable setzen/lesen
	int SetVariable(short deviceID, char *name, char *val);
	int GetVariable(short deviceID, char *name, char *val);

	// Info senden
	int ShowInfo(short deviceID, unsigned char maschine, unsigned char time, unsigned char font, unsigned char audio, short flags, char *text);

	// Zutrittsbuchung durchfuehren
	int Entrance2Identification(short deviceID, char *TM, char *Id, char *Pin, char *status, int &statusLength);
	int Entrance2IdentificationOK(short deviceID);
	// LED und Relais auf Tuermodul schalten
	int Entrance2OnlineAction(short deviceID, char *TM, unsigned int mask, unsigned char type, unsigned char duration);

	// Eine Benachrichtigung melden
	int Notify(short deviceID, unsigned char type, int priority, char *timestamp, char *params);
	int NotifyRecord(short deviceID, int priority);

	// Zu verwendender Zeichensatz bei Anzeige von Message oder Info
	int SetTypeFont(short deviceID, unsigned char type);

	//#################################
	// CallRoutinen

	// Ausfuehren von Befehlen
	virtual int Call(short module, short func, unsigned char *data, int &count);

};

#endif
