// DFFileStudio.h: Schnittstelle fr die Klasse CDFFileStudio.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_DFFILESTUDIO_H__49311412_E6B3_413F_A3C7_0A3F25DF431C__INCLUDED_)
#define AFX_DFFILESTUDIO_H__49311412_E6B3_413F_A3C7_0A3F25DF431C__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "../Common/aes.h"
#include "../Common/crc.h"
#include "../Common/DFSoftwareVersion.h"
#include <stdio.h>

// Angabe ueber die maximal bekannte Firmware-Version.
#define MAX_FIRMWARE1_VERSION "04.01.09.99"
#define MAX_FIRMWARE2_VERSION "04.02.03.99"

#define AESHEADER_CHECK 0xDFDFDFDF ///< StructAESHeader.check: Pruefwert fuer Gueltigkeit (Versionskennung) 
// Einen Dezimalwert 99 in Hexwert 0x99 wandeln.
//#define AESHEADER_DEC2VERSION(x) (x + (x / 10) * 6)
// Einen Hexwert 0x99 in Dezimalwert 99 wandeln.
//#define AESHEADER_VERSION2DEC(x) (x - (x / 16) * 6)
#define AESHEADER_VERSION(master, major, minor, build) ((master & 0xFF) << 24 | (major & 0xFF) << 16 | (minor & 0xFF) << 8 | (build & 0xFF))
#define AESHEADER_VERSION_UNSUPPORTED                      AESHEADER_VERSION(99, 99, 99, 99)

// Flags fuer den Dateiheader
#define AESHEADER_FLAG_IGNORE_SHOULD_STUDIO_VERSION        0x00000001
#define AESHEADER_FLAG_IGNORE_SHOULD_LIBRARY_VERSION       0x00000002
#define AESHEADER_FLAG_IGNORE_SHOULD_FIRMWARE1_VERSION     0x00000004
#define AESHEADER_FLAG_IGNORE_SHOULD_FIRMWARE2_VERSION     0x00000008

// Flags fuer die Loadroutine Pruefung auf Versionen
#define AESHEADER_IGNORE_MIN_STUDIO_VERSION            0x00000001
#define AESHEADER_IGNORE_SHOULD_STUDIO_VERSION         0x00000002
#define AESHEADER_IGNORE_MIN_LIBRARY_VERSION           0x00000004
#define AESHEADER_IGNORE_SHOULD_LIBRARY_VERSION        0x00000008

// Wird der Header verwendet, wird eine Firmware 04.00.xx nicht mehr unterstuetzt!

typedef struct StructAESHeader {
	unsigned int check;                      ///< Muss immer mit AESHEADER_CHECK belegt werden. 
                                           ///< Wird herangezogen um zu erkennen ob eine Headerstruktur in der Datei vorliegt.
	unsigned int size;                       ///< Groesse der Struktur sizeof(TAESHeader). 
                                           ///< Ist auch gleichzeitig eine Art Datei-Version. Die Struktur kann derzeit maximal (EEP_SIZE - EEP_SIZE_SU) gross werden.
  unsigned int flags;                      ///< Flags fuer z. B. "Gelesen mit DFComDLL, jedoch noch nicht bearbeitet"
	unsigned int studioVersion;              ///< Studioversion mit der die Datei zuletzt abgespeichert wurde.
                                           ///< Wurde die Datei ueber die DLL ausgelesen liegt hier keine Angabe vor!
  // Mindestversionen
	unsigned int minStudioVersion;           ///< Mindestversion des einzusetzenden Studios, um alle enthaltenen Daten zu unterstuetzen.
	unsigned int shouldStudioVersion;        ///< Sollteversion des einzusetzenden Studios, um alle enthaltenen Daten zu unterstuetzen.
  unsigned int minLibraryVersion;          ///< Mindestversion der zu verwendenden DFComDLL fuer die korrekte Uebertragung der Setupdaten.
  unsigned int shouldLibraryVersion;       ///< Sollteversion der zu verwendenden DFComDLL fuer die korrekte Uebertragung der Setupdaten.
  unsigned int minFirmware1Version;        ///< Mindestversion der zu verwendenden Firmware 04.01.xx.xx, um alle gewaehlten Einstellungen zu unterstuetzen.
  unsigned int shouldFirmware1Version;     ///< Sollteversion der zu verwendenden Firmware 04.01.xx.xx, um alle gewaehlten Einstellungen zu unterstuetzen.
  unsigned int minFirmware2Version;        ///< Mindestversion der zu verwendenden Firmware 04.02.xx.xx, um alle gewaehlten Einstellungen zu unterstuetzen.
  unsigned int shouldFirmware2Version;     ///< Sollteversion der zu verwendenden Firmware 04.02.xx.xx, um alle gewaehlten Einstellungen zu unterstuetzen.
  // CRC der Dateibereiche
  unsigned int crcEep;                     ///< CRC ueber Eep-Bereich.
  unsigned int crcEepExtended;             ///< CRC ueber EepExtended-Bereich (dort liegt auch Header jedoch mit eigenem CRC)
  unsigned int crcFlash;                   ///< CRC ueber Flash-Bereich.
  unsigned int crcListFile;                ///< CRC ueber die Listendateiangaben.
  unsigned int crcConfig;                  ///< CRC ueber die Config-Struktur.
  unsigned int crcFlashExtended;           ///< CRC ueber FlashExtended-Bereich.
  unsigned int crcListEntranceFile;        ///< CRC ueber die Listendateiangaben der ZK1.
  unsigned int crcListEntrance2File;       ///< CRC ueber die Listendateiangaben der ZK2.
  unsigned int crcListExtendedFile;        ///< CRC ueber die Zusaetzlichen Listendateiangaben.
	//...
	unsigned int crc;                        ///< CRC ueber alle Kopfdaten
} TAESHeader;

class CDFFileStudio  
{
public:
	CDFFileStudio();
	virtual ~CDFFileStudio();

	void SetDefault();
  void SetDefaultHeader();

	int Load(const char *fileName, int *error, int flags);
	int Save(const char *fileName, int *error);
	unsigned int GetFlashSetupSize();
	int NeededVersion();
	void ChangeOrderLong(unsigned char *Buf);

  inline int IsHeader() {return (m_fileHeader && m_fileHeader->check == AESHEADER_CHECK); }
  inline TAESHeader& GetHeader() { return (TAESHeader&)*m_fileHeader; }

  void SetStudioVersion(const char *version) {
    m_studioVersion.Parse(version);
    m_fileHeader->studioVersion = AESHEADER_VERSION(m_studioVersion.GetMaster(), m_studioVersion.GetMajor(), m_studioVersion.GetMinor(), m_studioVersion.GetBuild());
  }
  // Anpassung auf eventuell benoetigte Mindestversion.
  void NeedStudioVersion(const char *version) {
    CDFSoftwarebuildVersion minStudioVersion(m_fileHeader->minStudioVersion);
    CDFSoftwarebuildVersion minVersion(version);
    if (minStudioVersion < minVersion) {
      m_fileHeader->minStudioVersion = AESHEADER_VERSION(minVersion.GetMaster(), minVersion.GetMajor(), minVersion.GetMinor(), minVersion.GetBuild());
    }
  }
  void ShouldStudioVersion(const char *version) {
    CDFSoftwarebuildVersion studioVersion(m_fileHeader->shouldStudioVersion);
    CDFSoftwarebuildVersion shouldVersion(version);
    if (studioVersion < shouldVersion) {
      m_fileHeader->shouldStudioVersion = AESHEADER_VERSION(shouldVersion.GetMaster(), shouldVersion.GetMajor(), shouldVersion.GetMinor(), shouldVersion.GetBuild());
    }
  }
  void SetLibraryVersion(const char *version) {
    m_libraryVersion.Parse(version);
  }
  void NeedLibraryVersion(const char *version) {
    CDFSoftwarebuildVersion minLibraryVersion(m_fileHeader->minLibraryVersion);
    CDFSoftwarebuildVersion minVersion(version);
    if (minLibraryVersion < minVersion) {
      m_fileHeader->minLibraryVersion = AESHEADER_VERSION(minVersion.GetMaster(), minVersion.GetMajor(), minVersion.GetMinor(), minVersion.GetBuild());
    }
  }
  void ShouldLibraryVersion(const char *version) {
    CDFSoftwarebuildVersion libraryVersion(m_fileHeader->shouldLibraryVersion);
    CDFSoftwarebuildVersion shouldVersion(version);
    if (libraryVersion < shouldVersion) {
      m_fileHeader->shouldLibraryVersion = AESHEADER_VERSION(shouldVersion.GetMaster(), shouldVersion.GetMajor(), shouldVersion.GetMinor(), shouldVersion.GetBuild());
    }
  }
  void ShouldFirmware1Version(const char *version) {
    CDFSoftwarebuildVersion firmwareVersion(m_fileHeader->shouldFirmware1Version);
    CDFSoftwarebuildVersion shouldVersion(version);
    if (firmwareVersion < shouldVersion) {
      m_fileHeader->shouldFirmware1Version = AESHEADER_VERSION(shouldVersion.GetMaster(), shouldVersion.GetMajor(), shouldVersion.GetMinor(), shouldVersion.GetBuild());
    }
  }
  void NeedFirmware1Version(const char *version) {
    CDFSoftwarebuildVersion minFirmwareVersion(m_fileHeader->minFirmware1Version);
    CDFSoftwarebuildVersion minVersion(version);
    if (minFirmwareVersion < minVersion) {
      m_fileHeader->minFirmware1Version = AESHEADER_VERSION(minVersion.GetMaster(), minVersion.GetMajor(), minVersion.GetMinor(), minVersion.GetBuild());
    }
  }
  void UnsupportFirmware1Version() {
    m_fileHeader->minFirmware1Version = AESHEADER_VERSION_UNSUPPORTED;
  }
  void ShouldFirmware2Version(const char *version) {
    CDFSoftwarebuildVersion firmwareVersion(m_fileHeader->shouldFirmware2Version);
    CDFSoftwarebuildVersion shouldVersion(version);
    if (firmwareVersion < shouldVersion) {
      m_fileHeader->shouldFirmware2Version = AESHEADER_VERSION(shouldVersion.GetMaster(), shouldVersion.GetMajor(), shouldVersion.GetMinor(), shouldVersion.GetBuild());
    }
  }
  void NeedFirmware2Version(const char *version) {
    CDFSoftwarebuildVersion minFirmwareVersion(m_fileHeader->minFirmware2Version);
    CDFSoftwarebuildVersion minVersion(version);
    if (minFirmwareVersion < minVersion) {
      m_fileHeader->minFirmware2Version = AESHEADER_VERSION(minVersion.GetMaster(), minVersion.GetMajor(), minVersion.GetMinor(), minVersion.GetBuild());
    }
  }
  void UnsupportFirmware2Version() {
    m_fileHeader->minFirmware2Version = AESHEADER_VERSION_UNSUPPORTED;
  }
  void NeedFirmwareVersion(const char *version) {
    CDFSoftwarebuildVersion fwVersion(version);
    // Pruefung auf 04.01.xx
    if (fwVersion >= "04.01.01.00" && fwVersion < "04.02.00.00") {

      // Mindestversion der Firmware 04.01.xx.yy setzen.
      NeedFirmware1Version(version);
    } else if (fwVersion >= "04.02.00.00" && fwVersion < "04.03.00.00") {

      // Mindestversion der Firmware 04.02.xx.yy setzen.
      NeedFirmware2Version(version);
    }
  }
  void ShouldFirmwareVersion(const char *version) {
    CDFSoftwarebuildVersion fwVersion(version);
    // Pruefung auf 04.01.xx
    if (fwVersion >= "04.01.01.00" && fwVersion < "04.02.00.00") {

      // Empfohlene Firmware 04.01.xx.yy eintragen.
      ShouldFirmware1Version(version);
    } else if (fwVersion >= "04.02.00.00" && fwVersion < "04.03.00.00") {

      // Empfohlene Firmware 04.02.xx.yy pruefen?
      ShouldFirmware2Version(version);
    }
  }
  int IsSupportedFirmwareVersion(const char *version) {
    CDFSoftwarebuildVersion fwVersion(version);
    // Pruefung auf 04.01.xx
    if ((fwVersion >= "04.01.01.00" && fwVersion <= MAX_FIRMWARE1_VERSION) ||
        (fwVersion >= "04.02.00.00" && fwVersion <= MAX_FIRMWARE2_VERSION)) {
          return 1;
    }

    return 0;
  }
  int ValidFirmwareVersion(CDFSoftwarebuildVersion &fwVersion);

  // CRC uber Dateibereiche
  unsigned int GetCrcEep();                ///< CRC ueber Eep-Bereich.
  unsigned int GetCrcEepExtended();        ///< CRC ueber Eep-Bereich.
  unsigned int GetCrcFlash();              ///< CRC ueber Flash-Bereich.
  unsigned int GetCrcListFile();           ///< CRC ueber die Listendateiangaben.
  unsigned int GetCrcConfig();             ///< CRC ueber die Config-Struktur.
  unsigned int GetCrcFlashExtended();      ///< CRC ueber FlashExtended-Bereich.
  unsigned int GetCrcListEntranceFile();   ///< CRC ueber die Listendateiangaben der ZK1.
  unsigned int GetCrcListEntrance2File();  ///< CRC ueber die Listendateiangaben der ZK2.
  unsigned int GetCrcListExtendedFile();   ///< CRC ueber die Zusaetzlichen Listendateiangaben.

public:
  CDFSoftwarebuildVersion m_studioVersion;
  CDFSoftwarebuildVersion m_libraryVersion;
	TAESHeader *m_fileHeader;
	TConfig m_config;
	unsigned char *m_configMem;
	unsigned char *m_eep;
  unsigned char *m_header;
	unsigned char *m_flash;
	unsigned char *m_flashExtended;
	unsigned int m_flashSizeSU2;

	// Pfadangaben der Listen / Zusatzpfade
	char m_listFileName[MAX_DATENSATZ_BESCHREIBUNG][260];
	char m_listEntranceFileName[4][260];
	char m_listEntrance2FileName[MAX_DATENSATZ_BESCHREIBUNG][260];
	char m_strExtendedFilePath[MAX_DATENSATZ_BESCHREIBUNG][260];
};

#endif // !defined(AFX_DFFILESTUDIO_H__49311412_E6B3_413F_A3C7_0A3F25DF431C__INCLUDED_)
