#ifndef _EXTENDED_FLASH_H_
#define _EXTENDED_FLASH_H_

// ########################################################
// Erweiterter Flash

///////////////////////////////////////
// Struct: StructExtendedFlash

#define EXTENDED_CHECK 0xAAAAAAAE  ///< StructAE3Extended.ulCheck: Pruefwert fuer Gueltigkeit (Versionskennung) 

#define EXTF_EMPTY     0x00000000  ///< StructAE3Extended.ulFlags: Nicht vorhanden.
#define EXTF_NEW       0x00000001  ///< StructAE3Extended.ulFlags: Struktur angelegt.
#define EXTF_DOWNLOAD  0x00000010  ///< StructAE3Extended.ulFlags: Aus Geraet ausgelesene Daten.

/** Diese Struktur enthaelt die Kopfdaten des erweiterten Speichers.

  Diese Struktur enthaelt die Adressen der einzelnen Bloecke. Sie ist die erste und einzige Struktur ihrer art
  im Erweiterten Speicherblock (16kByte).
*/
typedef struct StructExtendedFlash {
	unsigned long ulCheck;             ///< muss immer mit EXTENDED_CHECK belegt werden.
	unsigned long ulSize;              ///< groesse der Stuktur sizeof(TExtendedFlash)
	unsigned long pulIOModule;         ///< Zeiger auf IOModul - Struktur
	unsigned long pulIODigital;        ///< Zeiger auf die 8 TDigitalInput - Strukturen des IOModul
	unsigned long pulMDEDigital;       ///< Zeiger auf die 4 TDigitalInput - Strukturen fuer intern.
	unsigned long pulEKArray;          ///< Zeiger auf PointerArray der Eingabeketten fuer MDE
	unsigned long pulFreeScope;        ///< Zeiger auf naechsten freien Speicherbereich im ExtendedFlashMem
	unsigned long pulEntranceTables;   ///< Tabellen fuer Zutrittskontrolle
	unsigned long pulEKArrayEntrance;  ///< Zeiger auf PointerArray der Eingabeketten fuer Zutrittskontrolle
	unsigned long pulValueGVx;         ///< Zeiger auf Struktur mit Standardwerten der GVs
	unsigned long pulEntrance2Tables;  ///< Tabellen fuer Zutrittskontrolle
	unsigned long pulEKArrayEntrance2; ///< Zeiger auf PointerArray der Eingabeketten fuer Zutrittskontrolle
	unsigned long pulExtended;         ///< Zeiger auf Struktur fuer Erweiterte Funktionalitaeten
	unsigned long pulMDEDigitalExt;    ///< Zeiger auf die weiteren 4 TDigitalInput - Strukturen fuer intern.
	unsigned long pulMDEAnalog;        ///< Zeiger auf die 2 TAnalogInput - Strukturen fuer intern.
	unsigned char tmp[36];             ///< Reserviert. Auf 100 Byte Strukturgroesse bringen.
	unsigned long ulFlags;             ///< Flags fuer Struktur und Erweiterten Speicher
} TExtendedFlash;

///////////////////////////////////////
// Struct: StructHardwareInfo

/** Diese Struktur enthaelt die Informationen zum Geraet.

  Diese Struktur ist z. Z. nur Definiert.
*/
typedef struct StructHardwareInfo {
	unsigned char typeDevice;
	unsigned char typeBoard;
	unsigned char componentRS485_0;
	unsigned char componentTCPIP;
	unsigned char typeModemMC;
	unsigned char componentRS485_1;
	unsigned char typeTransponder;
	unsigned char typeDisplay;
	unsigned char typeKeypad;
	unsigned char typeMMC;
	unsigned char typeGPS;
	unsigned char componentUSB;
	unsigned char typeRadio;
	unsigned char typeDCF77;
	unsigned long componentInAnalog;
	unsigned long componentInDigital;
	unsigned long componentOutDigital;
	unsigned char typeFingerprint;
	unsigned char typeExternalReader;
	unsigned char optionIOModule;
	unsigned char optionTBAdaption;
	unsigned char tmp[70];
} TDFHardwareInfo;

///////////////////////////////////////
// Struct: StructAE3IOModule

#define MAX_UNIT_TXT	5 ///< StructAE3IOModule.szUnitU: Maximallaenge des Einheitssuffix mit 0 terminie.

/** Diese Struktur enthaelt die Informationen zum IO-Module.

  Diese Struktur ist im erweiterten Speicherblock abgelegt.
*/
typedef struct StructIOModule {
	
	// Spannungseingaenge
	unsigned char ucActiveU;       ///< Flags welcher Eingang aktiv.
	char szNameU[8][MAX_NAME];     ///< Bezeichner der Eingaenge.
	float fMinU[8];                ///< Minimalwert fuer Berechnung.
	float fMaxU[8];                ///< Maximalwert fuer Berechnung.
	char szUnitU[8][MAX_UNIT_TXT]; ///< Einheit - suffix der Berechnung.
	unsigned char ucOnScreenU;     ///< Flags ob Ausgabe auf Display erlaubt.

	// Stromeingaenge
	unsigned char ucActiveI;       ///< Flags welcher Eingang aktiv.
	char szNameI[8][MAX_NAME];     ///< Bezeichner der Eingaenge
	float fMinI[8];                ///< Minimalwert fuer Berechnung.
	float fMaxI[8];                ///< Maximalwert fuer Berechnung.
	char szUnitI[8][MAX_UNIT_TXT]; ///< Einheit - suffix der Berechnung.
	unsigned char ucOnScreenI;     ///< Flags ob Ausgabe auf Display erlaubt.

} TIOModule;


///////////////////////////////////////
// Struct: StructDigitalInput

/** Diese Struktur enthaelt die Informationen fuer die Behandlung eines digitalen Einganges.

  Diese Struktur ist im erweiterten Speicherblock abgelegt.
*/
typedef struct StructDigitalInput {

	unsigned char ucUseAs;         ///< Verwendung als Start/Stop, Zaehler etc..
	unsigned char ucStartLowHigh;  ///< Auszufuehrende Eingabekette bei Flankenwechsel low -> high
	unsigned char ucStartHighLow;  ///< Auszufuehrende Eingabekette bei Flankenwechsel high -> low
	unsigned short uiCountDivisor; ///< Bei Verwendung als Zaehler, dessen Teiler
	unsigned short uiCycle;        ///< Bei Verwendung als Zaehler, dessen Zyklus.
	unsigned short uiTimeOut;      ///< Bei Verwendung als Zaehler mit Start/Stop, dessen Timeout.
	unsigned long ulCountDivisor32;///< Bei Verwendung als Zaehler, dessen Teiler als 32bit
	unsigned short uiDebounceHigh; ///< Endprellzeit fuer Flankenwechsel Low-High
	unsigned short uiDebounceLow;  ///< Endprellzeit fuer Flankenwechsel High-Low
	unsigned char ucInvokeInputChainCountDivisor; ///< Auszufuehrende Eingabekette bei erreichtem Zaehler
	unsigned char ucInvokeInputChainCycle;        ///< Auszufuehrende Eingabekette bei erreichtem Zyklus

	unsigned char ucTmp[2];       ///< Reserviert.

} TDigitalInput;

///////////////////////////////////////
// Struct: StructAnalogInput

/** Diese Struktur enthaelt die Informationen fuer die Behandlung eines analogen Einganges.

  Diese Struktur ist im erweiterten Speicherblock abgelegt.
*/
/*
typedef struct StructAnalogInput {

	unsigned char useAs;            ///< Eingangsverwendung als ...
	char name[MAX_NAME];            ///< Bezeichner der Eingaenge.
	float min;                      ///< Minimalwert fuer Berechnung.
	float max;                      ///< Maximalwert fuer Berechnung.
	char unit[MAX_UNIT_TXT];        ///< Einheit - suffix der Berechnung.
	unsigned char show;             ///< Flags ob Ausgabe auf Display erlaubt.
	unsigned char invokeInputChain; ///< Auszufuehrende Eingabekette bei Statuswechsel
	float limitLowerMessage;        ///< Wert fuer die untere Meldegrenze
	float limitLowerWarning;        ///< Wert fuer die untere Warngrenze
	float limitUpperMessage;        ///< Wert fuer die obere Meldegrenze
	float limitUpperWarning;        ///< Wert fuer die obere Warngrenze
	unsigned char sampleRate;       ///< Abtastrate fuer den analogen Eingang

	unsigned char tmp[10];          ///< Reserviert.

} TAnalogInput;
*/
typedef struct StructAnalogInput {

	unsigned char useAs;               ///< Eingangsverwendung als ...
	unsigned char show;                ///< Flags ob Ausgabe auf Display erlaubt.
	unsigned char sampleRate;          ///< Abtastrate fuer den analogen Eingang
	char name[MAX_NAME];               ///< Bezeichner der Eingaenge.
	char unit[MAX_UNIT_TXT];           ///< Einheit - suffix der Berechnung.
	float minLimit;                    ///< Minimalwert fuer Berechnung.
	float maxLimit;                    ///< Maximalwert fuer Berechnung.
	float minInput;                    ///< Minimalwert fuer Eingangsspannung.
	float maxInput;                    ///< Maximalwert fuer Eingangsspannung.
	float limitUpperAlarm;             ///< Wert fuer die obere Alarmgrenze
	float limitUpperWarning;           ///< Wert fuer die obere Warngrenze
	float limitLowerWarning;           ///< Wert fuer die untere Warngrenze
	float limitLowerAlarm;             ///< Wert fuer die untere Alarmgrenze
	unsigned char limitTolerance;      ///< Grenzwertabweichung
	unsigned char invokeInputChain[5]; ///< Auszufuehrende Eingabekette bei Statuswechsel

	unsigned char tmp[27];             ///< Reserviert.

} TAnalogInput;


///////////////////////////////////////
// Struct: StructValueGVx

/** Diese Struktur enthaelt die ggf. zu setzenden Standardwerte der globalen Variablen.

  Diese Struktur ist im erweiterten Speicherblock abgelegt.
*/
typedef struct StructValueGVx {
	unsigned char ucFlags;            ///< Bitfeld gibt an welche Variablen gesetzt werden sollen.
	char szValue[8][MAX_LENGTH_FELD]; ///< Zu setzender Standardwert der Variablen.
} TValueGVx;


///////////////////////////////////////
// Struct: StructFingerprint

/** Diese Struktur enthaelt die Einstellparameter fuer ein Fingerprint-Modul

  Diese Struktur ist im erweiterten Speicherblock abgelegt.
*/
typedef struct StructFingerprint {
	unsigned char method;             ///< Methode des Fingerprints
	unsigned char segment;            ///< Startsegment der abgelegten Fingertemplates auf Ausweis
	unsigned char securityLevel;      ///< Sicherheitslevel; der sich aus Falschrueckweisungrate et Falschakzeptanzrate ergibt.
	float securityFRR;                ///< Falschrueckweisungsrate
	float securityFAR;                ///< Falschakzeptanzrate
	unsigned char flags;              ///< Bit(0) Templateformat => 0=DIN V66400; 1=Idencom; Bit(1-7) frei.
	unsigned char quality;            ///< Mindestbildqualitaet beim scannen
	unsigned char minutiaeCount;      ///< Mindestanzahl der Minutiae beim scannen
	char tmp[6];                      ///< Reserve fuer evntl. Erweiterungen
} TFingerprint;

// Zusatztypen 
typedef struct StructFT {
	unsigned long pidHi;
	unsigned long pidLo;
	unsigned char fid;
	unsigned char quality;
	unsigned char count;
	unsigned char data[50][3];
} structFingertemplate;

typedef struct StructId {
	char pid[21];
	char fid[4];
} structId;

///////////////////////////////////////
// Struct: StructTBAdaption

/** Diese Struktur enthaelt die Einstellparameter fuer eine Timeboyadaption an MasterIV-Geraete

  Diese Struktur ist im erweiterten Speicherblock abgelegt.
*/
typedef struct StructTBAdaption {
	unsigned char method;               ///< Methode der Adaption
	unsigned char count;                ///< Anzahl der Steckplaetze bei Mehrfachdocking
	unsigned char records;              ///< Datenablage auf MasterIV oder Timeboy
	unsigned char invokeInputChainPush; ///< Auszufuehrende Eingabekette, wenn Timeboy gesteckt wird
	unsigned char invokeInputChainPull; ///< Auszufuehrende Eingabekette, wenn Timeboy gezogen wird
	char tmp[15];                       ///< Reserve fuer evntl. Erweiterungen
} TTBAdaption;

///////////////////////////////////////
// Struct: StructGPRS

/** Diese Struktur enthaelt die Einstellparameter fuer eine GPRS-Kommunikation

  Diese Struktur ist im erweiterten Speicherblock abgelegt.
*/
typedef struct StructGPRS {
	unsigned char method;               ///< Methode der GPRS-Kommunikation
	unsigned char invokeInputChain;     ///< Auszufuehrende Eingabekette, wenn Alive-Timeout abgelaufen
	char tmp[8];                        ///< Reserve fuer evntl. Erweiterungen
} TGPRS;


///////////////////////////////////////
// Struct: StructEnergieOption

/** Diese Struktur enthaelt die Einstellparameter fuer die EnergieOption zum Modbil-MasterIV.

  Diese Struktur ist im erweiterten Speicherblock abgelegt.
*/
typedef struct StructEnergieOption {
	unsigned char mode;                 ///< Aktivierung der Option
	unsigned char digitalIn;            ///< Eingabekette fuer das An/Aus der Zuendung
	unsigned short timeout;             ///< Timout, nach dem die Module nach einem An, eingeschaltet werden
	unsigned char invokeRelease;        ///< Eingabekette zur Fahrzeugfreigabe.
	unsigned char invokeGPS;            ///< Eingabekette bei gueltigen GPS-Daten.
	unsigned char digitalOut;           ///< Zu schaltendes Relais fuer Selbstabschaltung
	unsigned char flags;                ///< Flags fuer die Energieoption
	                                    ///< Bit0: Einzel-(0) oder Dauerfreischaltung(1).
	                                    ///< Bit1-7: unused
	unsigned char tmp[8];               ///< Reserve
} TEnergieOption;

///////////////////////////////////////
// Struct: StructTimer

/** Diese Struktur enthaelt die Einstellparameter fuer die Timer
	Die Struktur ist im extendet Speicherblock abgelegt.
*/
typedef struct StructTimer {
	unsigned char ucUseAs;          ///< Verwendung als Timer in Eingabeketten etc..
	unsigned short uiCycle;         ///< Bei Verwendung als Timer, dessen Zyklus.
	unsigned short uiTimeOut;       ///< Bei Verwendung als Timer, dessen Timeout.
	unsigned char invokeInputChain; ///< Auszufuehrende Eingabekette, wenn Timeout abgelaufen, oder danach der Zyklus abgelaufen.

	unsigned char tmp[10];          ///< Reserve
} TTimer;

///////////////////////////////////////
// Struct: StructExtended

/** Diese Struktur enthaelt zusaetzliche Informationen.

  Diese Struktur ist im erweiterten Speicherblock abgelegt. 
  In dieser Struktur werden alle weiteren Informationen abgelegt.
*/
typedef struct StructExtended {

	unsigned short doubleInputTime;  ///< Zeit fuer "Doppelte Eingabe" PZE1/PZE2 Modus in Sekunden
	unsigned short confirmationTime; ///< Zeit fuer "Besttigungsfelder" in Sekunden
	// Sommer/Winterzeitumschaltung
	unsigned char daylightActive;    ///< Aktivieren der Sommer-/Winterzeitumschaltung (daylight saving)
	unsigned char daylight1Month;    ///< Monat in dem Umgestellt wird (1 - 12)
	unsigned char daylight1Week;     ///< Monatswoche in der Umgestellt wird (1 - 5, wobei 5 letzte im Monat bedeutet)
	unsigned char daylight1Day;      ///< Tag der Umschaltung (1 - 7, wobei 1 = Montag)
	unsigned char daylight1Hour;     ///< Stunde fuer Umschaltung (0 - 23)
	unsigned char daylight2Month;    ///< Monat in dem Umgestellt wird (1 - 12)
	unsigned char daylight2Week;     ///< Monatswoche in der Umgestellt wird (1 - 5, wobei 5 letzte im Monat bedeutet)
	unsigned char daylight2Day;      ///< Tag der Umschaltung (1 - 7, wobei 1 = Montag)
	unsigned char daylight2Hour;     ///< Stunde fuer Umschaltung (0 - 23)
	char transponderPassword32[9];   ///< Passwort fuer Hitag2 (32bit) in ASCII (Hex) abgelegt.
	char transponderPassword48[13];  ///< Passwort fuer Mifare (48bit) in ASCII (Hex) abgelegt.
	unsigned char phgAESKey[16];     ///< Schluessel fuer Kommunikationsverschluesselung der PHG Leser
	TFingerprint fingerprint;        ///< Datenstruktur zum Fingerprint
	TTBAdaption adaption;            ///< Anbindungsstruktur von Timboys an MasterIV
	TGPRS gprs;                      ///< Struktur fuer Zusatzangaben bezueglich GPRS, wie z. B. Alive.
	TEnergieOption energieOption;    ///< Struktur fuer Energieoption bei Mobil-MasterIV
	TTimer timer[4];				 ///< Struktur fuer Timer in der Signalverarbeitung (vorgesehen sind 4 Timer a 16 Byte)
	unsigned char tmp[843];          ///< Frei fuer Erweiterungen
} TExtended;

#endif
